require 'logger'


module Retag
  class Logger < ::Logger

    def shift(spaces: 2, &block)
      if block_given?
        with_shift(spaces: spaces, &block)
      else
        @shift ||= 0
        @shift += spaces
      end
    end

    def unshift(spaces: 2)
      @shift ||= 0
      @shift -= spaces
    end

    def with_shift(spaces: 2)
      shift(spaces: spaces)
      yield
    ensure
      unshift(spaces: spaces)
    end

    def format_message(severity, datetime, progname, msg)
      super(severity, datetime, progname, "#{' ' * (@shift || 0)}#{msg}")
    end

  end
end

