
module Retag
  class Repo

    include Utils
    attr_reader :repo, :branch, :revision, :name

    def initialize(repo, branch)
      @repo = repo
      @branch = branch
      @gitlab = Gitlab.client
    end

    def tag!(tag, message: nil)
      begin
        result = @gitlab.tag(name, tag)
        if result.to_h.dig('commit', 'short_id').to_s == revision
          # Success: tag with same revision already exists
          return true
        end
      rescue Gitlab::Error::NotFound
        # Success: there is no such tag
      end
      
      @gitlab.create_tag(name, tag, revision, message)
    end

    def name
      @name ||= @repo.split(':').last.gsub('.git', '')
    end

    def revision
      @revision ||= begin
        # "^{}" means  the object could be a tag. https://mirrors.edge.kernel.org/pub/software/scm/git/docs/gitrevisions.html#_specifying_revisions 
        # Example:
        # git ls-remote git@gir.example.com:repo.git | grep release-latest
        # 174cf2b6685df563e59c5cd0d367996fcf24f0ae        refs/tags/release-latest       <-- annotated tag itself
        # 47208e276cd277896887417a383e05973278438c        refs/tags/release-latest^{}    <-- WE NEED: original commit tag is references to
        rev_name_pair = remote_refs(repo, branch+"^{}") # try to find commit tag is references to
        rev_name_pair ||= remote_refs(repo, branch)     # try to find simple branch or commit

        raise "There is no '#{branch}' revision at #{repo}" if rev_name_pair.nil?

        rev, _name = rev_name_pair.split
        rev[0..7]
      end
    end

  private

    def remote_refs repo, branch
      cmd!("git ls-remote #{repo} #{branch}", capture: true).split("\n").select{|line| line.strip.end_with?("/#{branch}") }.first
    end

  end
end

